@@ -24,23 +24,36 @@ int output_bufferN;
 static char *output_buffer_raw, *output_buffer_raw_bak;
 static int output_buffer_rawN;
 gboolean test_mode;
+#if WIN32
+int last_input_method;
+#endif
 
+void init_gtab(int inmdno);
 
 char current_method_type()
 {
   if (!current_CS)
+#if UNIX
     return inmd[default_input_method].method_type;
+#else
+  {
+    if (!last_input_method)
+		last_input_method = default_input_method;
+	return inmd[last_input_method].method_type;
+  }
+#endif
   return inmd[current_CS->in_method].method_type;
 }
 
 
 #if WIN32
-void win32_FakeKey(UINT vk);
+void win32_FakeKey(UINT vk, bool key_pressed);
 #endif
 void send_fake_key_eve(KeySym key)
 {
 #if WIN32
-  win32_FakeKey(key);
+  win32_FakeKey(key, true);
+  win32_FakeKey(key, false);
 #else
   KeyCode kc = XKeysymToKeycode(dpy, key);
   XTestFakeKeyEvent(dpy, kc, True, CurrentTime);
@@ -297,8 +310,8 @@ void hide_win_anthy();
 void hide_in_win(ClientState *cs)
 {
   if (!cs) {
-#if DEBUG
-    dbg("hide_in_win: ic is null");
+#if 0
+    dbg("hide_in_win: ic is null\n");
 #endif
     return;
   }
@@ -357,7 +370,7 @@ gboolean force_show;
 void show_in_win(ClientState *cs)
 {
   if (!cs) {
-#if DEBUG
+#if 0
     dbg("show_in_win: ic is null");
 #endif
     return;
@@ -408,7 +421,7 @@ void move_in_win(ClientState *cs, int x, int y)
     y = gcin_root_y;
   }
 
-#if DEBUG || 0
+#if 0
   dbg("move_in_win %d %d\n",x, y);
 #endif
 #if 1
@@ -458,7 +471,7 @@ void getRootXY(Window win, int wx, int wy, int *tx, int *ty)
 #if WIN32
   POINT pt;
   pt.x = wx; pt.y = wy;
-  ClientToScreen(win, &pt);
+  ClientToScreen((HWND)win, &pt);
   *tx = pt.x; *ty=pt.y;
 #else
   Window ow;
@@ -501,7 +514,7 @@ void move_IC_in_win(ClientState *cs)
 #else
    if (inpwin) {
      RECT rect;
-	 GetClientRect(inpwin, &rect);
+	 GetClientRect((HWND)inpwin, &rect);
      if (inpx >= rect.right)
        inpx = rect.right - 1;
      if (inpy >= rect.bottom)
@@ -512,7 +525,7 @@ void move_IC_in_win(ClientState *cs)
    int tx,ty;
    getRootXY(inpwin, inpx, inpy, &tx, &ty);
 
-#if DEBUG
+#if 0
    dbg("move_IC_in_win inpxy:%d,%d txy:%d,%d\n", inpx, inpy, tx, ty);
 #endif
 
@@ -537,7 +550,7 @@ void update_in_win_pos()
     winx++; winy++;
 
     Window inpwin = current_CS->client_win;
-#if DEBUG
+#if 0
     dbg("update_in_win_pos\n");
 #endif
     if (inpwin) {
@@ -636,10 +649,17 @@ void init_state_chinese(ClientState *cs)
   cs->im_state = GCIN_STATE_CHINESE;
   set_tsin_pho_mode0(cs);
   if (!cs->in_method)
+#if UNIX
     init_in_method(default_input_method);
+#else
+  if (!last_input_method)
+	last_input_method = default_input_method;
+  init_in_method(last_input_method);
+#endif
 }
 
 gboolean output_gbuf();
+void update_win_kbm();
 
 void toggle_im_enabled()
 {
@@ -673,10 +693,16 @@ void toggle_im_enabled()
 #endif
       current_CS->im_state = GCIN_STATE_DISABLED;
 
+      update_win_kbm();
+
 #if TRAY_ENABLED
       disp_tray_icon();
 #endif
     } else {
+      if (!current_method_type())
+        init_gtab(current_CS->in_method);
+
+
       init_state_chinese(current_CS);
       reset_current_in_win_xy();
 #if 1
@@ -687,6 +713,8 @@ void toggle_im_enabled()
       show_in_win(current_CS);
 #endif
 
+      update_win_kbm();
+
 #if TRAY_ENABLED
       disp_tray_icon();
 #endif
@@ -697,6 +725,7 @@ void get_win_gtab_geom();
 void get_win0_geom();
 void get_win_pho_geom();
 void get_win_anthy_geom();
+void get_win_int_geom();
 
 void update_active_in_win_geom()
 {
@@ -711,6 +740,7 @@ void update_active_in_win_geom()
       break;
 #endif
     case method_type_INT_CODE:
+	  get_win_int_geom();
       break;
 #if USE_ANTHY
     case method_type_ANTHY:
@@ -723,7 +753,7 @@ void update_active_in_win_geom()
   }
 }
 
-extern GtkWidget *gwin_pho, *gwin0, *gwin_gtab;
+extern GtkWidget *gwin_pho, *gwin0, *gwin_gtab, *gwin_int;
 int anthy_visible();
 
 gboolean win_is_visible()
@@ -738,7 +768,7 @@ gboolean win_is_visible()
       return gwin0 && GTK_WIDGET_VISIBLE(gwin0);
 #endif
     case method_type_INT_CODE:
-      break;
+	  return gwin_int && GTK_WIDGET_VISIBLE(gwin_int);
 #if USE_ANTHY
     case method_type_ANTHY:
       return anthy_visible();
@@ -764,6 +794,8 @@ void toggle_half_full_char()
     return;
 #endif
 
+  check_CS();
+
   if (!gcin_shift_space_eng_full) {
     current_CS->b_half_full_char = 0;
     tss.tsin_half_full=0;
@@ -771,7 +803,6 @@ void toggle_half_full_char()
     return;
   }
 
-  check_CS();
 
 //  dbg("toggle_half_full_char\n");
 
@@ -797,7 +828,6 @@ void toggle_half_full_char()
 //  dbg("half full toggle\n");
 }
 
-void init_gtab(int inmdno);
 void init_inter_code();
 void init_tab_pp(gboolean init);
 void init_tab_pho();
@@ -809,7 +839,7 @@ void hide_win_kbm();
 extern gboolean win_sym_enabled;
 int init_win_anthy();
 void show_win_kbm();
-void update_win_kbm();
+void set_gtab_input_method_name(char *s);
 
 gboolean init_in_method(int in_no)
 {
@@ -878,8 +908,14 @@ gboolean init_in_method(int in_no)
         win_kbm_inited = 1;
         show_win_kbm();
       }
+
+      set_gtab_input_method_name(inmd[in_no].cname);
       break;
   }
+#if WIN32
+  if (current_CS && current_CS->in_method != last_input_method)
+	  last_input_method = current_CS->in_method;
+#endif
 
 #if TRAY_ENABLED
   disp_tray_icon();
@@ -897,10 +933,10 @@ gboolean init_in_method(int in_no)
 static void cycle_next_in_method()
 {
   int i;
-
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   for(i=0; i < MAX_GTAB_NUM_KEY; i++) {
     int v = ((current_CS->in_method + i) % MAX_GTAB_NUM_KEY) + 1;
     if (!(gcin_flags_im_enabled & (1<<v)))
@@ -1065,8 +1101,13 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
     if (kidx <= 0)
       return FALSE;
 
+    if (!inmd[kidx].cname)
+      return FALSE;
+
     current_CS->im_state = GCIN_STATE_CHINESE;
+#if WIN32
     if (!test_mode)
+#endif
       init_in_method(kidx);
 
     return TRUE;
@@ -1325,6 +1366,8 @@ int gcin_FocusOut(ClientState *cs)
 int tsin_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_comp_len);
 int anthy_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor);
 int gtab_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor, int *sub_comp_len);
+int int_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_comp_len);
+int pho_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_comp_len);
 
 int gcin_get_preedit(ClientState *cs, char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_comp_len)
 {
@@ -1344,8 +1387,9 @@ empty:
 
   switch(current_method_type()) {
     case method_type_PHO:
+		return pho_get_preedit(str, attr, cursor, sub_comp_len);
     case method_type_INT_CODE:
-      goto empty;
+		return int_get_preedit(str, attr, cursor, sub_comp_len);
 #if USE_TSIN
     case method_type_TSIN:
       return tsin_get_preedit(str, attr, cursor, sub_comp_len);
