@@ -10,7 +10,11 @@
 static GdkPixbuf *pixbuf, *pixbuf_ch;
 static PangoLayout* pango;
 static GtkWidget *da;
+#if !GTK_CHECK_VERSION(2,90,6)
 static GdkGC *gc;
+#else
+static cairo_t *gc;
+#endif
 GdkWindow *tray_da_win;
 static EggTrayIcon *egg_tray_icon;
 
@@ -30,6 +34,7 @@ static void get_text_w_h(char *s, int *w, int *h)
 
 static void draw_icon()
 {
+//  dbg("draw_icon\n");
 #if 0
   return;
 #endif
@@ -51,15 +56,35 @@ static void draw_icon()
 
   GdkColor color_fg;
 
+
+//  dbg("wh %d,%d\n", dw,dh);
+
   gdk_color_parse("black", &color_fg);
+#if !GTK_CHECK_VERSION(2,90,6)
   gdk_gc_set_rgb_fg_color(gc, &color_fg);
+#else
+  gc = gdk_cairo_create (tray_da_win);
+  gdk_cairo_set_source_color (gc, &color_fg);
+#endif
 
   if (pix) {
     int ofs = (dh - gdk_pixbuf_get_height (pix))/2;
+#if !GTK_CHECK_VERSION(2,90,6)
     gdk_draw_pixbuf(tray_da_win, NULL, pix, 0, 0, 0, ofs, -1, -1, GDK_RGB_DITHER_NORMAL, 0, 0);
+#else
+    gdk_cairo_set_source_pixbuf (gc, pix, 0, ofs);
+    cairo_paint (gc);
+    cairo_destroy (gc);
+#endif
   } else {
     get_text_w_h(inmd[current_CS->in_method].cname, &w, &h);
+#if !GTK_CHECK_VERSION(2,90,6)
     gdk_draw_layout(tray_da_win, gc, 0, 0, pango);
+#else
+    cairo_move_to (gc, 0, 0);
+    pango_cairo_show_layout (gc, pango);
+    cairo_destroy (gc);
+#endif
   }
 
   if (current_CS) {
@@ -70,33 +95,67 @@ static void draw_icon()
         current_CS->im_state == GCIN_STATE_CHINESE) {
       static char full[] = "全";
       get_text_w_h(full,  &w, &h);
+#if !GTK_CHECK_VERSION(2,90,6)
       gdk_draw_layout(tray_da_win, gc, dw - w, dh - h, pango);
+#else
+      cairo_move_to (gc, dw - w, dh - h);
+      pango_cairo_show_layout (gc, pango);
+      cairo_destroy (gc);
+#endif
     }
 
     if (current_CS->im_state == GCIN_STATE_ENG_FULL) {
       static char efull[] = "A全";
       get_text_w_h(efull,  &w, &h);
+#if !GTK_CHECK_VERSION(2,90,6)
       gdk_draw_layout(tray_da_win, gc, 0, 0, pango);
+#else
+      cairo_move_to (gc, 0, 0);
+      pango_cairo_show_layout (gc, pango);
+      cairo_destroy (gc);
+#endif
     }
 #if USE_TSIN
     if ((current_method_type()==method_type_TSIN||current_method_type()==method_type_ANTHY) && current_CS->im_state == GCIN_STATE_CHINESE && !tsin_pho_mode()) {
       static char efull[] = "ABC";
       gdk_color_parse("blue", &color_fg);
+#if !GTK_CHECK_VERSION(2,90,6)
       gdk_gc_set_rgb_fg_color(gc, &color_fg);
+#else
+      gc = gdk_cairo_create (tray_da_win);
+      gdk_cairo_set_source_color (gc, &color_fg);
+#endif
 
       get_text_w_h(efull,  &w, &h);
+#if !GTK_CHECK_VERSION(2,90,6)
       gdk_draw_layout(tray_da_win, gc, 0, 0, pango);
+#else
+      cairo_move_to (gc, 0, 0);
+      pango_cairo_show_layout (gc, pango);
+      cairo_destroy (gc);
+#endif
     }
 #endif
   }
 
   gdk_color_parse("red", &color_fg);
+#if !GTK_CHECK_VERSION(2,90,6)
   gdk_gc_set_rgb_fg_color(gc, &color_fg);
+#else
+  gc = gdk_cairo_create (tray_da_win);
+  gdk_cairo_set_source_color (gc, &color_fg);
+#endif
 
   if (gb_output) {
     static char sim[] = "简";
     get_text_w_h(sim,  &w, &h);
+#if !GTK_CHECK_VERSION(2,90,6)
     gdk_draw_layout(tray_da_win, gc, 0, dh - h, pango);
+#else
+    cairo_move_to (gc, 0, dh - h);
+    pango_cairo_show_layout (gc, pango);
+    cairo_destroy (gc);
+#endif
   }
 
 }
@@ -104,6 +163,7 @@ static void draw_icon()
 gboolean create_tray(gpointer data);
 void update_tray_icon()
 {
+//  dbg("update_tray_icon\n");
   if (!gcin_status_tray)
     return;
 
@@ -117,6 +177,7 @@ void get_icon_path(char *iconame, char fname[]);
 
 void load_tray_icon()
 {
+//  dbg("load_tray_icon\n");
   if (!gcin_status_tray)
     return;
 
@@ -131,12 +192,30 @@ void load_tray_icon()
   if (iconame)
     get_icon_path(iconame, fname);
 
+#if GTK_CHECK_VERSION(2,17,7)
+  GtkAllocation dwdh;
+  gtk_widget_get_allocation(da, &dwdh);
+  int dw = dwdh.width, dh = dwdh.height;
+#else
+    int dw = da->allocation.width, dh = da->allocation.height;
+#endif
+
+  if (!pixbuf || gdk_pixbuf_get_width (pixbuf) != dw || gdk_pixbuf_get_height (pixbuf) != dh) {
+    char icon_fname[128];
+    get_icon_path(GCIN_TRAY_PNG, icon_fname);
+    GError *err = NULL;
+//    dbg("icon_name %s\n", icon_fname);
+    pixbuf = gdk_pixbuf_new_from_file_at_size(icon_fname, dw, dh, &err);
+    if (!pixbuf)
+      p_err("cannot load file %s", icon_fname);
+  }
+
 #if 0
   dbg("fname %x %s\n", fname, fname);
 #endif
   if (!fname[0]) {
     if (pixbuf_ch)
-      gdk_pixbuf_unref(pixbuf_ch);
+      g_object_unref(pixbuf_ch);
 
     pixbuf_ch = NULL;
     if (pixbuf_ch_fname)
@@ -147,16 +226,10 @@ void load_tray_icon()
     pixbuf_ch_fname = strdup(fname);
 
     if (pixbuf_ch)
-      gdk_pixbuf_unref(pixbuf_ch);
+      g_object_unref(pixbuf_ch);
 
+    dbg("ch %s\n", fname);
     GError *err = NULL;
-#if GTK_CHECK_VERSION(2,17,7)
-    GtkAllocation dwdh;
-    gtk_widget_get_allocation(da, &dwdh);
-    int dw = dwdh.width, dh = dwdh.height;
-#else
-    int dw = da->allocation.width, dh = da->allocation.height;
-#endif
     pixbuf_ch = gdk_pixbuf_new_from_file_at_size(fname, dw, dh, &err);
   }
 
@@ -185,6 +258,8 @@ void cb_tog_gcb(GtkCheckMenuItem *checkmenuitem, gpointer dat);
 void kbm_toggle_(GtkCheckMenuItem *checkmenuitem, gpointer dat);
 extern int win_kbm_on;
 
+void cb_inmd_menu(GtkCheckMenuItem *checkmenuitem, gpointer dat);
+
 static MITEM mitems[] = {
   {N_("設定"), GTK_STOCK_PREFERENCES, exec_gcin_setup_, NULL},
   {N_("重新執行gcin"), NULL, restart_gcin, NULL},
@@ -194,7 +269,8 @@ static MITEM mitems[] = {
 #endif
   {N_("正->簡體"), NULL, cb_trad2sim, NULL},
   {N_("簡->正體"), NULL, cb_sim2trad, NULL},
-  {N_("小鍵盤"), NULL, kbm_toggle_, &win_kbm_on},
+  {N_("選擇輸入法"), NULL, cb_inmd_menu, NULL},
+  {N_("小鍵盤"), NULL, kbm_toggle_, NULL},
   {N_("简体输出"), NULL, cb_trad_sim_toggle_, &gb_output},
   {NULL, NULL, NULL, NULL}
 };
@@ -255,10 +331,13 @@ gboolean cb_expose(GtkWidget *da, GdkEventExpose *event, gpointer data)
   if (!da)
     create_tray(NULL);
 
+//  dbg("cb_expose\n");
+
   draw_icon();
   return FALSE;
 }
 
+GdkGC *gdk_gc_new (GdkDrawable *drawable);
 
 gboolean create_tray(gpointer data)
 {
@@ -283,20 +362,15 @@ gboolean create_tray(gpointer data)
                     G_CALLBACK (tray_button_press_event_cb), NULL);
 
   GError *err = NULL;
+  if (pixbuf)
+    g_object_unref(pixbuf);
 
   char icon_fname[128];
   get_icon_path(GCIN_TRAY_PNG, icon_fname);
-
-  if (pixbuf)
-    gdk_pixbuf_unref(pixbuf);
   pixbuf = gdk_pixbuf_new_from_file(icon_fname, &err);
-
   int pwidth = gdk_pixbuf_get_width (pixbuf);
   int pheight = gdk_pixbuf_get_height (pixbuf);
 
-  if (!pixbuf)
-    p_err("cannot load file %s", icon_fname);
-
   da =  gtk_drawing_area_new();
   g_signal_connect (G_OBJECT (event_box), "destroy",
                     G_CALLBACK (gtk_widget_destroyed), &da);
@@ -314,13 +388,6 @@ gboolean create_tray(gpointer data)
     return FALSE;
   }
 
-#if 0
-  Window xwin = GDK_WINDOW_XID(tray_da_win);
-  XWindowAttributes att;
-  XGetWindowAttributes(dpy, xwin, &att);
-  dbg("www %d %d\n", att.width, att.height);
-#endif
-
   PangoContext *context=gtk_widget_get_pango_context(da);
   PangoFontDescription* desc=pango_context_get_font_description(context);
 
@@ -339,7 +406,9 @@ gboolean create_tray(gpointer data)
   dbg("context %x %x\n", pango_layout_get_context(pango), context);
   dbg("font %x %x\n",pango_layout_get_font_description(pango), desc);
 #endif
+#if !GTK_CHECK_VERSION(2,90,6)
   gc = gdk_gc_new (tray_da_win);
+#endif
   return FALSE;
 }
 
