@@ -1462,6 +1462,19 @@ gboolean tsin_page_down()
   return TRUE;
 }
 
+void open_select_pho()
+{
+  if (tss.c_idx==tss.c_len) {
+    get_sel_phrase_end();
+  } else
+    get_sel_phrase();
+
+  get_sel_pho();
+  tss.sel_pho=1;
+  tss.current_page = 0;
+  disp_current_sel_page();
+}
+
 int feedkey_pp(KeySym xkey, int kbstate)
 {
   char ctyp=0;
@@ -1548,6 +1561,9 @@ int feedkey_pp(KeySym xkey, int kbstate)
           return 1;
         }
      case XK_Home:
+#if UNIX
+     case XK_KP_Home:
+#endif
         close_selection_win();
         if (!tss.c_len)
           return 0;
@@ -1556,6 +1572,9 @@ int feedkey_pp(KeySym xkey, int kbstate)
         drawcursor();
         return 1;
      case XK_End:
+#if UNIX
+     case XK_KP_End:
+#endif
         close_selection_win();
         if (!tss.c_len)
           return 0;
@@ -1564,8 +1583,14 @@ int feedkey_pp(KeySym xkey, int kbstate)
         drawcursor();
         return 1;
      case XK_Left:
+#if UNIX
+     case XK_KP_Left:
+#endif
         return cursor_left();
      case XK_Right:
+#if UNIX
+     case XK_KP_Right:
+#endif
         return cursor_right();
      case XK_Caps_Lock:
         if (caps_eng_tog) {
@@ -1597,6 +1622,9 @@ tab_phrase_end:
         }
         return 0;
      case XK_Delete:
+#if UNIX
+     case XK_KP_Delete:
+#endif
         return cursor_delete();
      case XK_BackSpace:
         close_selection_win();
@@ -1668,6 +1696,9 @@ tab_phrase_end:
 
         return 1;
      case XK_Up:
+#if UNIX
+     case XK_KP_Up:
+#endif
        if (!tss.sel_pho) {
          if (tss.c_len && tss.c_idx == tss.c_len) {
            int idx = tss.c_len-1;
@@ -1686,6 +1717,10 @@ tab_phrase_end:
        tsin_page_up();
        return 1;
      case XK_Prior:
+#if UNIX
+     case XK_KP_Prior:
+#endif
+     case XK_KP_Subtract:
        return tsin_page_up();
      case XK_space:
        if (!tss.c_len && !poo.ityp3_pho && !poo.typ_pho[0] && !poo.typ_pho[1] && !poo.typ_pho[2]
@@ -1708,6 +1743,9 @@ tab_phrase_end:
        if (!tsin_pho_mode())
            goto asc_char;
      case XK_Down:
+#if UNIX
+     case XK_KP_Down:
+#endif
        if (!poo.ityp3_pho && (poo.typ_pho[0]||poo.typ_pho[1]||poo.typ_pho[2]) && xkey==XK_space) {
          ctyp=3;
          kno=0;
@@ -1725,21 +1763,17 @@ change_char:
          return 1;
 
        if (!tss.sel_pho) {
-         if (tss.c_idx==tss.c_len) {
-           get_sel_phrase_end();
-         } else
-           get_sel_phrase();
-
-         get_sel_pho();
-         tss.sel_pho=1;
-         tss.current_page = 0;
-         disp_current_sel_page();
+         open_select_pho();
        } else {
          tsin_page_down();
        }
 
        return 1;
      case XK_Next:
+#if UNIX
+     case XK_KP_Next:
+#endif
+     case XK_KP_Add:
        return tsin_page_down();
      case '\'':  // single quote
        if (phkbm.phokbm[xkey][0].num)
@@ -2195,6 +2229,7 @@ int tsin_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_
 #if _DEBUG && 0
   dbg("tsin_get_preedit\n");
 #endif
+
   for(i=0; i<tss.c_len; i++) {
     if (tn>=GCIN_PREEDIT_MAX_STR-4*CH_SZ-1)
       goto fin;
