@@ -3,7 +3,7 @@
 #include "win-sym.h"
 #include "gst.h"
 
-GtkWidget *gwin0;
+GtkWidget *gwin0 = NULL;
 extern GtkWidget *gwin1;
 Window xwin0;
 extern Display *dpy;
@@ -23,6 +23,7 @@ static struct {
   GtkWidget *vbox;
   GtkWidget *label;
   GtkWidget *line;
+  int x;
 } chars[MAX_PH_BF_EXT];
 
 
@@ -65,6 +66,16 @@ void set_label_font_size();
    gtk_main(), the coodinate of the widget is sometimes invalid.
    We use pre-create to overcome this bug.
 */
+
+void drawcursor();
+void open_select_pho();
+static void mouse_char_callback( GtkWidget *widget,GdkEventButton *event, gpointer data)
+{
+  tss.c_idx = GPOINTER_TO_INT(data);
+  drawcursor();
+  open_select_pho();
+}
+
 static void create_char(int index)
 {
   int i;
@@ -74,13 +85,19 @@ static void create_char(int index)
   GdkColor color_bg;
   gdk_color_parse(tsin_phrase_line_color, &color_bg);
 
-  for(i=index; i<=index+1 && i < MAX_PH_BF_EXT; i++) {
+  i = index;
+//  for(i=index; i<=index+1 && i < MAX_PH_BF_EXT; i++)
+  {
     if (chars[i].vbox)
-      continue;
+      return;
 
+    GtkWidget *event_box = gtk_event_box_new();
+    chars[i].vbox = event_box;
+    g_signal_connect (G_OBJECT (event_box), "button-press-event",  G_CALLBACK (mouse_char_callback), GINT_TO_POINTER(index));
+
+    gtk_box_pack_start (GTK_BOX (hbox_edit), event_box, FALSE, FALSE, 0);
     GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox_edit), vbox, FALSE, FALSE, 0);
-    chars[i].vbox = vbox;
+    gtk_container_add(GTK_CONTAINER(event_box), vbox);
 
     GtkWidget *label = gtk_label_new(NULL);
     gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
@@ -97,8 +114,8 @@ static void create_char(int index)
     if (gcin_win_color_use)
       gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &fg);
 
-    gtk_widget_show(vbox);
-    gtk_widget_show(label);
+    gtk_widget_show_all(event_box);
+    gtk_widget_hide(separator);
   }
 }
 
@@ -149,15 +166,16 @@ void disp_char(int index, char *ch)
   if (win_x + win_xl >= dpy_xl)
     move_win0(dpy_xl - win_xl, win_y);
 
-  gtk_widget_show(chars[index].vbox);
-  gtk_widget_show(label);
+  gtk_widget_show_all(chars[index].vbox);
+  gtk_widget_hide(chars[index].line);
 }
 
 void hide_char(int index)
 {
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   if (!chars[index].vbox)
     return;
   gtk_label_set_text(GTK_LABEL(chars[index].label), "");
@@ -168,10 +186,10 @@ void hide_char(int index)
 void clear_chars_all()
 {
   int i;
-
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   for(i=0; i < MAX_PH_BF_EXT; i++) {
     hide_char(i);
   }
@@ -257,13 +275,12 @@ void clr_in_area_pho_tsin()
    disp_tsin_pho(i, " ");
 }
 
-void get_widget_xy(GtkWidget *win, GtkWidget *widget, int *rx, int *ry)
+int get_widget_xy(GtkWidget *win, GtkWidget *widget, int *rx, int *ry)
 {
   if (!win && !widget)
     p_err("get_widget_xy err");
 
-//  gtk_widget_show_all(widget);
-  gdk_flush();
+//  gdk_flush();
 
   GtkRequisition sz;
   gtk_widget_size_request(widget, &sz);
@@ -290,6 +307,7 @@ void get_widget_xy(GtkWidget *win, GtkWidget *widget, int *rx, int *ry)
 
   *rx = win_x + wx;
   *ry = win_y + wy;
+  return wx;
 }
 
 void getRootXY(Window win, int wx, int wy, int *tx, int *ty);
@@ -318,14 +336,20 @@ void disp_tsin_select(int index)
     getRootXY(current_CS->client_win, current_CS->spot_location.x, current_CS->spot_location.y, &x, &y);
   } else {
     int i;
-    for(i=0;i<=index;i++) {
-      if (!chars[i].vbox)
+	// bug in GTK, widget position is wrong, repeat util find one
+    for(i=index;i>=0; i--) {
+#if _DEBUG
+      if (!chars[i].vbox || !chars[index].label)
         p_err("err found");
+#endif
+      gtk_widget_show_now(chars[i].label);
       gtk_widget_show(chars[i].vbox);
-	  gtk_widget_show(chars[i].label);
+      int tx = get_widget_xy(gwin0, chars[i].vbox, &x, &y);
+
+      if (tx>=0)
+        break;
+//	  dbg("%d] %d\n", i, tx);
     }
-    gtk_widget_show(gwin0);
-    get_widget_xy(gwin0, widget, &x, &y);
   }
   disp_selections(x, y);
 }
@@ -371,6 +395,7 @@ void compact_win0_x()
 
 static void compact_win0()
 {
+  if (! gwin0) return;
   max_yl = 0;
   gtk_window_resize(GTK_WINDOW(gwin0), MIN_X_SIZE, 16);
   raw_move(best_win_x, best_win_y);
@@ -479,7 +504,7 @@ void create_win0()
 {
   if (gwin0)
     return;
-#if _DEBUG
+#if _DEBUG && 0
   dbg("create_win0\n");
 #endif
   gwin0 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -579,7 +604,7 @@ static void create_win0_gui()
     gtk_widget_set_tooltip_text (button_pho, _(_L("左鍵符號，右鍵設定")));
 #else
     GtkTooltips *button_pho_tips = gtk_tooltips_new ();
-    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_pho_tips), button_pho, _(_L"左鍵符號，右鍵設定"),NULL);
+    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_pho_tips), button_pho, _(_L("左鍵符號，右鍵設定")),NULL);
 #endif
   }
 
@@ -632,10 +657,12 @@ void raise_tsin_selection_win();
 
 void show_win0()
 {
+#if WIN32
   if (test_mode)
     return;
+#endif
 
-#if _DEBUG
+#if _DEBUG && 0
 	dbg("show_win0 pop:%d in:%d for:%d \n", gcin_pop_up_win, tsin_has_input(), force_show);
 #endif
   create_win0();
